#pragma semicolon 1

#include <sourcemod>
#include <vip>

public Plugin:myinfo = 
{
	name	= "VIP Module (HP Regen)",
	author	= "wS",
	version = "1.1"
};

new const String:g_sItemName[]	= "hp_regen";
new g_ItemID;
new g_MaxHP[MAXPLAYERS + 1],	g_RegenHP[MAXPLAYERS + 1];
new Handle:g_hRegenTimer[MAXPLAYERS + 1]		= {INVALID_HANDLE, ...};
new g_RegenColor[4];
new g_OldColor[MAXPLAYERS + 1][4];

public OnPluginStart()
{
	if ((g_ItemID = VIP_RegisterItem(g_sItemName, true, true)) < 1)
		SetFailState("VIP_RegisterItem error");

	new Handle:cvar				= CreateConVar("vip_hp_regen_rgba", "0 255 0 255", "Цвет игрока во время регенерации (0 = стандарт)");
	wS_GetColor(cvar);
	HookConVarChange(cvar,		cvar_changed);

	HookEvent("player_hurt",	player_hurt,	EventHookMode_Post);
}

public OnPluginEnd()
{
	VIP_UnRegisterItem(g_sItemName);
}

public cvar_changed(Handle:cvar, const String:OldValue[], const String:NewValue[])
{
	wS_GetColor(cvar);
}

public OnClientConnected(client)
{
	g_RegenHP[client] 	= 0;
	g_MaxHP[client]   	= 0;
}

public OnClientDisconnect(client)
{
	wS_ClearTimer(g_hRegenTimer[client]);
}

public VIP_OnPlayerSpawn(client, team, bool:bot, bool:vip, const String:steamid[], const String:group[])
{
	g_MaxHP[client] = 0;
}

public VIP_OnClientAuthorized(client, const String:steamid[], bool:vip, const String:group[])
{
	if (vip) wS_Check(client);
}

public VIP_OnVipItemToggled(client, ItemID, const String:ItemName[], const String:ItemInfo[], bool:enabled)
{
	if (ItemID == g_ItemID)
	{
		g_RegenHP[client] = enabled ? StringToInt(ItemInfo) : 0;

		if (wS_ClearTimer(g_hRegenTimer[client]))
		{
			wS_SetColor(client, g_OldColor[client][0], g_OldColor[client][1], g_OldColor[client][2], g_OldColor[client][3]);
		}

		if (g_RegenHP[client] > 0 && IsPlayerAlive(client))
		{
			new curr_hp = GetEntProp(client, Prop_Send, "m_iHealth");
			if (g_MaxHP[client] < 1)
			{// (g_MaxHP[client] < 1	&&	(g_MaxHP[client] = curr_hp) < 100)
				g_MaxHP[client] = 100;
			}
			if (curr_hp < g_MaxHP[client]) 
			{
				wS_StartRegen(client);
			}
		}

		VIP_ShowMenu(client);
	}
}

public VIP_OnRemoved(client, const String:steamid[], const String:reason[])
{
	g_RegenHP[client] = 0;
}

public VIP_OnGroupChanged(client, const String:steamid[], const String:OldGroup[], const String:NewGroup[])
{
	g_RegenHP[client] = 0;
	wS_Check(client);
}

public VIP_OnAdded(client, const String:steamid[], const String:group[], min, const String:admin_steamid[])
{
	if (client > 0)
	{
		wS_Check(client);
	}
}

wS_Check(client)
{
	if (!VIP_ItemDisabled(g_sItemName) && VIP_GetItemToggledStatus(client, g_sItemName))
	{
		decl String:ItemInfo[MAX_ITEMINFO_LENGTH];
		if (VIP_GetClientItemInfo(client, g_sItemName, ItemInfo))
		{
			g_RegenHP[client] = StringToInt(ItemInfo);
		}
	}
}

stock bool:wS_ClearTimer(&Handle:t)
{
	if (t != INVALID_HANDLE)
	{
		KillTimer(t);
		t = INVALID_HANDLE;
		return true;
	}
	return false;
}

public player_hurt(Handle:event, const String:name[], bool:silent)
{
	new hp = GetEventInt(event, "health");
	if (hp > 0)
	{
		new client = GetClientOfUserId(GetEventInt(event, "userid"));
		if (g_RegenHP[client] > 0 && g_hRegenTimer[client] == INVALID_HANDLE)
		{
			if (g_MaxHP[client] < 1) g_MaxHP[client] = hp + GetEventInt(event, "dmg_health");
			wS_StartRegen(client);
		}
	}
}

wS_StartRegen(client)
{
	g_hRegenTimer[client] = CreateTimer(1.0, g_hRegenTimer_CallBack, client, TIMER_REPEAT);

	new offset = GetEntSendPropOffs(client, "m_clrRender");
	if (offset != -1)
	{
		for (new i = 0; i < 4; i++) g_OldColor[client][i] = GetEntData(client, offset + i, 1);
	}
	else
		g_OldColor[client] = {255, 255, 255, 255};

	if (g_RegenColor[0] > -1)
		wS_SetColor(client, g_RegenColor[0], g_RegenColor[1], g_RegenColor[2], g_RegenColor[3]);
}

public Action:g_hRegenTimer_CallBack(Handle:timer, any:client)
{
	if (IsPlayerAlive(client))
	{
		new hp = GetEntProp(client, Prop_Send, "m_iHealth");
		if (hp < g_MaxHP[client])
		{
			hp += g_RegenHP[client];
			if (hp > g_MaxHP[client])
			{
				hp = g_MaxHP[client];
			}
			
			SetEntProp(client, Prop_Send, "m_iHealth", hp);

			if (hp < g_MaxHP[client])
			{
				return Plugin_Continue;
			}
		}
	}
	g_hRegenTimer[client] = INVALID_HANDLE;
	wS_SetColor(client, g_OldColor[client][0], g_OldColor[client][1], g_OldColor[client][2], g_OldColor[client][3]);
	return Plugin_Stop;
}

wS_GetColor(Handle:cvar)
{
	decl String:str_rgba[35], String:buffer[4][9];
	GetConVarString(cvar, str_rgba, 35);

	if (TrimString(str_rgba) < 4 || ExplodeString(str_rgba, " ", buffer, 4, 9) < 4)
	{
		g_RegenColor[0] = -1;
		return;
	}

	decl value;
	for (new i = 0; i < 4; i++)
	{
		if ((value = StringToInt(buffer[i])) < 0 || value > 255)
		{
			g_RegenColor[0] = -1;
			return;
		}
		g_RegenColor[i] = value;
	}
}

wS_SetColor(entity, r, g, b, a)
{
	SetEntityRenderMode(entity, RENDER_TRANSCOLOR);
	SetEntityRenderColor(entity, r, g, b, a);
}